@@ -25,8 +25,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include "actions.h"
 #include "dlg-add-files.h"
 #include "dlg-add-folder.h"
@@ -35,18 +33,37 @@
 #include "dlg-open-with.h"
 #include "dlg-password.h"
 #include "dlg-prop.h"
-#include "main.h"
 #include "gtk-utils.h"
 #include "fr-window.h"
 #include "file-utils.h"
 #include "fr-process.h"
 #include "gconf-utils.h"
 #include "glib-utils.h"
+#include "main.h"
+#include "typedefs.h"
+
+
+typedef struct {
+	FrWindow  *window;
+	
+	GtkWidget *file_sel;
+	GtkWidget *combo_box;
+	GtkWidget *password;
+	GtkWidget *password_label;
+} SaveAsData;
 
 
 /* -- new archive -- */
 
 
+static void
+new_archive_dialog_destroy_cb (GtkWidget  *w,
+		               SaveAsData *data)
+{
+	g_free (data);
+}
+
+
 static void
 new_archive (GtkWidget *file_sel,
 	     FrWindow  *window, 
@@ -89,16 +106,14 @@ is_supported_extension (GtkWidget *file_sel,
 
 
 static char *
-get_full_path (GtkWidget *file_sel)
+get_full_path (SaveAsData *data)
 {
-	GtkWidget   *combo_box;
 	char        *full_path = NULL;
 	char        *path;
 	const char  *filename;
 	int          idx;
 
-	combo_box = g_object_get_data (G_OBJECT (file_sel), "fr_combo_box");
-	path = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (file_sel));
+	path = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->file_sel));
 
 	if ((path == NULL) || (*path == 0))
 		return NULL;
@@ -109,11 +124,11 @@ get_full_path (GtkWidget *file_sel)
 		return NULL;
 	}
 
-	idx = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
+	idx = gtk_combo_box_get_active (GTK_COMBO_BOX (data->combo_box));
 	if (idx > 0) {
 		const char *path_ext = fr_archive_utils__get_file_name_ext (path);
 		char       *default_ext = file_type_desc[save_type[idx-1]].ext;
-		if (strcmp_null_tollerant (path_ext, default_ext) != 0) {
+		if (strcmp_null_tolerant (path_ext, default_ext) != 0) {
 			full_path = g_strconcat (path, default_ext, NULL);
 			g_free (path);
 		}
@@ -126,19 +141,20 @@ get_full_path (GtkWidget *file_sel)
 
 
 static char *
-get_archive_filename_from_selector (FrWindow  *window,
-				    GtkWidget *file_sel)
+get_archive_filename_from_selector (SaveAsData *data)
 {
-	char *path = NULL;
-	char *dir;
+	char      *path = NULL;
+	GFile     *file, *dir;
+	GFileInfo *info;
+	GError    *err = NULL;
 
-	path = get_full_path (file_sel);
+	path = get_full_path (data);
 	if ((path == NULL) || (*path == 0)) {
 		GtkWidget *dialog;
 
 		g_free (path);
 
-		dialog = _gtk_error_dialog_new (GTK_WINDOW (file_sel),
+		dialog = _gtk_error_dialog_new (GTK_WINDOW (data->file_sel),
 						GTK_DIALOG_DESTROY_WITH_PARENT,
 						NULL,
 						_("Could not create the archive"),
@@ -149,14 +165,34 @@ get_archive_filename_from_selector (FrWindow  *window,
 		return NULL;
 	}
 
-	dir = remove_level_from_path (path);
-	if (! check_permissions (dir, R_OK | W_OK | X_OK)) {
+	file = g_file_new_for_uri (path);
+
+	dir = g_file_get_parent (file);
+	info = g_file_query_info (dir,
+				  G_FILE_ATTRIBUTE_ACCESS_CAN_READ "," 
+				  G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE "," 
+				  G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE,
+				  0, NULL, &err);
+	if (err != NULL) {
+		g_warning ("Failed to get permission for extraction dir: %s",
+			   err->message);
+		g_clear_error (&err);
+		g_object_unref (info);
+		g_object_unref (dir);
+		g_object_unref (file);
+		g_free (path);
+		return NULL;
+	}
+
+	if (! g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE)) {
 		GtkWidget *dialog;
 
-		g_free (dir);
+		g_object_unref (info);
+		g_object_unref (dir);
+		g_object_unref (file);
 		g_free (path);
 
-		dialog = _gtk_error_dialog_new (GTK_WINDOW (file_sel),
+		dialog = _gtk_error_dialog_new (GTK_WINDOW (data->file_sel),
 						GTK_DIALOG_DESTROY_WITH_PARENT,
 						NULL,
 						_("Could not create the archive"),
@@ -165,18 +201,17 @@ get_archive_filename_from_selector (FrWindow  *window,
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 		return NULL;
 	}
-	g_free (dir);
+	g_object_unref (info);
+	g_object_unref (dir);
 
 	/* if the user did not specify a valid extension use the filetype combobox current type
 	 * or tar.gz if automatic is selected. */
 	if (fr_archive_utils__get_file_name_ext (path) == NULL) {
-		GtkWidget *combo_box;
-		int        idx;
-		char      *new_path;
-		char      *ext = NULL;
+		int   idx;
+		char *new_path;
+		char *ext = NULL;
 
-		combo_box = g_object_get_data (G_OBJECT (file_sel), "fr_combo_box");
-		idx = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
+		idx = gtk_combo_box_get_active (GTK_COMBO_BOX (data->combo_box));
 		if (idx > 0)
 			ext = file_type_desc[save_type[idx-1]].ext;
 		else
@@ -188,13 +223,11 @@ get_archive_filename_from_selector (FrWindow  *window,
 
 	debug (DEBUG_INFO, "create/save %s\n", path);
 
-	if (path_is_file (path)) {
-		GtkWidget      *dialog;
-		int             r;
-		GnomeVFSResult  result;
+	if (uri_exists (path)) {
+		GtkWidget *dialog;
 
-		if (! is_supported_extension (file_sel, path)) {
-			dialog = _gtk_error_dialog_new (GTK_WINDOW (file_sel),
+		if (! is_supported_extension (data->file_sel, path)) {
+			dialog = _gtk_error_dialog_new (GTK_WINDOW (data->file_sel),
 							GTK_DIALOG_MODAL,
 							NULL,
 							_("Could not create the archive"),
@@ -206,127 +239,139 @@ get_archive_filename_from_selector (FrWindow  *window,
 			return NULL;
 		}
 
-		dialog = _gtk_message_dialog_new (GTK_WINDOW (file_sel),
-						  GTK_DIALOG_MODAL,
-						  GTK_STOCK_DIALOG_QUESTION,
-						  _("The archive already exists.  Do you want to overwrite it?"),
-						  NULL,
-						  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						  _("Overwrite"), GTK_RESPONSE_YES,
-						  NULL);
-
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
-		r = gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-
-		if (r != GTK_RESPONSE_YES) {
-			g_free (path);
-			return NULL;
-		}
-
-		result = gnome_vfs_unlink (path);
-		if (result != GNOME_VFS_OK) {
+		g_file_delete (file, NULL, &err);
+		if (err != NULL) {
 			GtkWidget *dialog;
-			dialog = _gtk_error_dialog_new (GTK_WINDOW (file_sel),
+			dialog = _gtk_error_dialog_new (GTK_WINDOW (data->file_sel),
 							GTK_DIALOG_DESTROY_WITH_PARENT,
 							NULL,
 							_("Could not delete the old archive."),
-							gnome_vfs_result_to_string (result));
+							err->message);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (GTK_WIDGET (dialog));
+			g_error_free (err);
 			g_free (path);
+			g_object_unref (file);
 			return NULL;
 		}
 	}
-
+	
+	g_object_unref (file);
+	
 	return path;
 }
 
 
 static void
-new_file_response_cb (GtkWidget *w,
-		      int        response,
-		      GtkWidget *file_sel)
+new_file_response_cb (GtkWidget  *w,
+		      int         response,
+		      SaveAsData *data)
 {
-	FrWindow *window;
-	char     *path;
-
-	window = g_object_get_data (G_OBJECT (file_sel), "fr_window");
+	char *path;
 
 	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
-		fr_archive_action_completed (window->archive,
+		fr_archive_action_completed (data->window->archive,
 					     FR_ACTION_CREATING_NEW_ARCHIVE,
 					     FR_PROC_ERROR_STOPPED,
 					     NULL);
-		gtk_widget_destroy (file_sel);
+		gtk_widget_destroy (data->file_sel);
 		return;
 	}
 
 	if (response == GTK_RESPONSE_HELP) {
-		show_help_dialog (GTK_WINDOW (file_sel), "file-roller-create");
+		show_help_dialog (GTK_WINDOW (data->file_sel), "file-roller-create");
 		return;
 	}
 
-	path = get_archive_filename_from_selector (window, file_sel);
+	path = get_archive_filename_from_selector (data);
 	if (path != NULL) {
-		new_archive (file_sel, window, path);
+		new_archive (data->file_sel, data->window, path);
 		g_free (path);
 	}
 }
 
 
+static void
+update_password_availability_for_ext (SaveAsData *data,
+				      const char *ext)
+{
+	int i;
+	
+	if (data->password == NULL) 
+		return;
+		
+	if (ext == NULL) {
+		gtk_widget_set_sensitive (data->password, FALSE);
+		gtk_widget_set_sensitive (data->password_label, FALSE);
+		return;
+	}
+				
+	for (i = 0; file_type_desc[i].id != FR_FILE_TYPE_NULL; i++) {
+		if (strcmp (file_type_desc[i].ext, ext) == 0) {
+			gtk_widget_set_sensitive (data->password, file_type_desc[i].supports_password);
+			gtk_widget_set_sensitive (data->password_label, file_type_desc[i].supports_password);
+			break;
+		}
+	}
+}
+
+
 static void
 filetype_combobox_changed_cb (GtkComboBox *combo_box,
-			      GtkWidget   *file_sel)
+			      SaveAsData  *data)
 {
 	int         idx;
-	const char *filename;
-	const char *ext, *newext;
-	char       *escaped_uri;
-	char       *unescaped_uri;	
-	char       *new_filename, *filename_noext;
-
-	idx = gtk_combo_box_get_active (combo_box) - 1;
-	if (idx < 0)
+	const char *uri, *basename;
+	const char *ext, *new_ext;
+	char       *basename_noext;
+	char       *new_basename;
+	char       *new_basename_uft8;
+	
+	uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->file_sel));
+	if (uri == NULL)
 		return;
-
-	escaped_uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (file_sel));
-	if (escaped_uri == NULL)
-		return;
-
-	unescaped_uri = gnome_vfs_format_uri_for_display (escaped_uri);
-	filename = file_name_from_path (unescaped_uri);
-	if (filename == NULL) {
-		g_free (unescaped_uri);
+	
+	ext = fr_archive_utils__get_file_name_ext (uri);
+	if (ext == NULL)
+		ext = "";
+	
+	idx = gtk_combo_box_get_active (GTK_COMBO_BOX (data->combo_box)) - 1;
+	if (idx < 0) {
+		if (data->password != NULL) {
+			gtk_widget_set_sensitive (data->password, TRUE);
+			gtk_widget_set_sensitive (data->password_label, TRUE);
+		} 
 		return;
 	}
 
-	ext = fr_archive_utils__get_file_name_ext (filename);
-	if (ext == NULL)
-		ext = "";
+	basename = file_name_from_path (uri);
+	basename_noext = g_strndup (basename, strlen (basename) - strlen (ext));
 
-	filename_noext = g_strndup (filename, strlen (filename) - strlen (ext));
-
-	newext = file_type_desc[save_type[idx]].ext;
-	new_filename = g_strconcat (filename_noext, newext, NULL);
-	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_sel), new_filename);
-
-	g_free (new_filename);
-	g_free (filename_noext);
-	g_free (unescaped_uri);
+	new_ext = file_type_desc[save_type[idx]].ext;
+	new_basename = g_strconcat (basename_noext, new_ext, NULL);
+	new_basename_uft8 = g_uri_unescape_string (new_basename, NULL);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (data->file_sel), new_basename_uft8);
+	update_password_availability_for_ext (data, new_ext);
+	
+	g_free (new_basename_uft8);
+	g_free (new_basename);
+	g_free (basename_noext);
 }
 
 
 void
-show_new_archive_dialog (FrWindow  *window,
+show_new_archive_dialog (FrWindow   *window,
 			 const char *archive_name)
 {
+	SaveAsData    *data;
 	GtkWidget     *file_sel;
 	GtkWidget     *hbox;
-	GtkWidget     *combo_box;
 	GtkFileFilter *filter;
 	int            i;
 
+	data = g_new0 (SaveAsData, 1);
+	data->window = window;
+	
 	file_sel = gtk_file_chooser_dialog_new (_("New"),
 						GTK_WINDOW (window),
 						GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -334,6 +379,7 @@ show_new_archive_dialog (FrWindow  *window,
 						GTK_STOCK_NEW, GTK_RESPONSE_OK,
 						GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 						NULL);
+	data->file_sel = file_sel;
 
 	gtk_dialog_set_default_response (GTK_DIALOG (file_sel), GTK_RESPONSE_OK);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), FALSE);
@@ -370,29 +416,29 @@ show_new_archive_dialog (FrWindow  *window,
 			    gtk_label_new (_("Archive type:")),
 			    FALSE, FALSE, 0);
 
-	combo_box = gtk_combo_box_new_text ();
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), _("Automatic"));
+	data->combo_box = gtk_combo_box_new_text ();
+	gtk_combo_box_append_text (GTK_COMBO_BOX (data->combo_box), _("Automatic"));
 	for (i = 0; save_type[i] != FR_FILE_TYPE_NULL; i++)
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
+		gtk_combo_box_append_text (GTK_COMBO_BOX (data->combo_box),
 					   _(file_type_desc[save_type[i]].name));
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
-	gtk_box_pack_start (GTK_BOX (hbox), combo_box, TRUE, TRUE, 0);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (data->combo_box), 0);
+	gtk_box_pack_start (GTK_BOX (hbox), data->combo_box, TRUE, TRUE, 0);
 	gtk_widget_show_all (hbox);
 
 	/**/
 
-	g_object_set_data (G_OBJECT (file_sel), "fr_window", window);
-	g_object_set_data (G_OBJECT (file_sel), "fr_combo_box", combo_box);
-	g_object_set_data (G_OBJECT (window), "fr_file_sel", file_sel);
-
 	g_signal_connect (G_OBJECT (file_sel),
 			  "response", 
 			  G_CALLBACK (new_file_response_cb), 
-			  file_sel);
-	g_signal_connect (G_OBJECT (combo_box),
+			  data);
+	g_signal_connect (G_OBJECT (data->combo_box),
 			  "changed", 
 			  G_CALLBACK (filetype_combobox_changed_cb),
-			  file_sel);
+			  data);
+	g_signal_connect (G_OBJECT (file_sel),
+			  "destroy", 
+			  G_CALLBACK (new_archive_dialog_destroy_cb),
+			  data);
 
 	gtk_window_set_modal (GTK_WINDOW (file_sel),TRUE);
 	gtk_widget_show_all (file_sel);
@@ -505,54 +551,57 @@ activate_action_open (GtkAction *action,
 
 
 static void
-save_file_destroy_cb (GtkWidget *w,
-		      GtkWidget *file_sel)
+save_file_destroy_cb (GtkWidget  *w,
+		      SaveAsData *data)
 {
+	g_free (data);
 }
 
 
 static void
-save_file_response_cb (GtkWidget *w,
-		       gint       response,
-		       GtkWidget *file_sel)
+save_file_response_cb (GtkWidget  *w,
+		       gint        response,
+		       SaveAsData *data)
 {
-	FrWindow *window;
-	char     *path;
+	char *path;
 
 	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
-		gtk_widget_destroy (file_sel);
+		gtk_widget_destroy (data->file_sel);
 		return;
 	}
 
 	if (response == GTK_RESPONSE_HELP) {
-		show_help_dialog (GTK_WINDOW (file_sel), "file-roller-convert-archive");
+		show_help_dialog (GTK_WINDOW (data->file_sel), "file-roller-convert-archive");
 		return;
 	}
 
-	window = g_object_get_data (G_OBJECT (file_sel), "fr_window");
-
-	path = get_archive_filename_from_selector (window, file_sel);
+	path = get_archive_filename_from_selector (data);
 	if (path == NULL)
 		return;
-
-	fr_window_archive_save_as (window, path);
-	gtk_widget_destroy (file_sel);
+		
+	fr_window_archive_save_as (data->window, path, gtk_entry_get_text (GTK_ENTRY (data->password)));
+	
+	gtk_widget_destroy (data->file_sel);
 	g_free (path);
 }
 
 
 void
 activate_action_save_as (GtkAction *action,
-			 gpointer   data)
+			 gpointer   callback_data)
 {
-	FrWindow      *window = data;
+	FrWindow      *window = callback_data;
 	GtkWidget     *file_sel;
+	GtkWidget     *table;
 	GtkWidget     *hbox;
-	GtkWidget     *combo_box;
+	GtkWidget     *label;
+	SaveAsData    *data;
 	GtkFileFilter *filter;
 	int            i;
-	const char    *escaped_uri = NULL;
-	char          *unescaped_uri = NULL;
+	const char    *password;
+
+	data = g_new0 (SaveAsData, 1);
+	data->window = window;
 
 	file_sel = gtk_file_chooser_dialog_new (_("Save"),
 						GTK_WINDOW (window),
@@ -561,15 +610,36 @@ activate_action_save_as (GtkAction *action,
 						GTK_STOCK_SAVE, GTK_RESPONSE_OK,
 						GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 						NULL);
+	data->file_sel = file_sel;
+	
 	gtk_dialog_set_default_response (GTK_DIALOG (file_sel), GTK_RESPONSE_OK);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), FALSE);
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (file_sel), TRUE);
 	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (file_sel), fr_window_get_open_default_dir (window));
 
 	if (fr_window_get_archive_uri (window)) {
-		escaped_uri = fr_window_get_archive_uri (window);
-		unescaped_uri = gnome_vfs_format_uri_for_display (escaped_uri);
-		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_sel), file_name_from_path (unescaped_uri));
-		g_free (unescaped_uri);
+		const char *uri;
+		GFile      *file;
+		GFileInfo  *info;
+		GError     *err = NULL;
+
+		uri = fr_window_get_archive_uri (window);
+		file = g_file_new_for_uri (uri);
+		info = g_file_query_info (file,
+				G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
+				0, NULL, &err);
+
+		if (err != NULL) {
+			g_warning ("Failed to get display name for uri %s: %s", uri, err->message);
+			g_clear_error (&err);
+		} 
+		else {
+			gtk_file_chooser_set_current_name (
+					GTK_FILE_CHOOSER (file_sel),
+					g_file_info_get_display_name (info));
+		}
+		g_object_unref (info);
+		g_object_unref (file);
 	}
 
 	filter = gtk_file_filter_new ();
@@ -584,41 +654,73 @@ activate_action_save_as (GtkAction *action,
 	gtk_file_filter_add_pattern (filter, "*");
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_sel), filter);
 
+	data->password = gtk_entry_new (); 
+	gtk_entry_set_visibility (GTK_ENTRY (data->password), FALSE);
+	
 	/**/
 
-	hbox = gtk_hbox_new (FALSE, 12);
-	gtk_box_pack_start (GTK_BOX (hbox),
-			    gtk_label_new (_("Archive type:")),
-			    FALSE, FALSE, 0);
+	table = gtk_table_new (2, 2, FALSE);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 
-	combo_box = gtk_combo_box_new_text ();
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), _("Automatic"));
+	/* archive type */
+
+	label = gtk_label_new_with_mnemonic (_("Archive _type:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
+			  (GtkAttachOptions) (GTK_FILL),
+			  (GtkAttachOptions) (0), 0, 0);
+
+	data->combo_box = gtk_combo_box_new_text ();
+	gtk_combo_box_append_text (GTK_COMBO_BOX (data->combo_box), _("Automatic"));
 	for (i = 0; save_type[i] != FR_FILE_TYPE_NULL; i++)
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
+		gtk_combo_box_append_text (GTK_COMBO_BOX (data->combo_box),
 					   _(file_type_desc[save_type[i]].name));
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
-	gtk_box_pack_start (GTK_BOX (hbox), combo_box, TRUE, TRUE, 0);
-	gtk_widget_show_all (hbox);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (data->combo_box), 0);
+	gtk_table_attach (GTK_TABLE (table), data->combo_box, 1, 2, 0, 1,
+			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+			  (GtkAttachOptions) (0), 0, 0);
 
-	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (file_sel), hbox);
+	/* password */
 
+	hbox = gtk_hbox_new (FALSE, 0);
+
+	data->password_label = label = gtk_label_new_with_mnemonic (_("_Encrypt with password:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+	
+	gtk_box_pack_start (GTK_BOX (hbox), data->password, FALSE, TRUE, 0);
+	
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
+			  (GtkAttachOptions) (GTK_FILL),
+			  (GtkAttachOptions) (0), 0, 0);
+	gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, 1, 2,
+			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+			  (GtkAttachOptions) (0), 0, 0);
+	
+	gtk_widget_show_all (table);
+	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (file_sel), table);
+	
+	/* set the default data */
+	
+	password = fr_window_get_password (window);
+	if (password == NULL)
+		password = "";
+	gtk_entry_set_text (GTK_ENTRY (data->password), password);
+	
 	/**/
 
-	g_object_set_data (G_OBJECT (file_sel), "fr_window", window);
-	g_object_set_data (G_OBJECT (file_sel), "fr_combo_box", combo_box);
-
 	g_signal_connect (G_OBJECT (file_sel),
 			  "response", 
 			  G_CALLBACK (save_file_response_cb), 
-			  file_sel);
+			  data);
 	g_signal_connect (G_OBJECT (file_sel),
 			  "destroy", 
 			  G_CALLBACK (save_file_destroy_cb),
-			  file_sel);
-	g_signal_connect (G_OBJECT (combo_box),
+			  data);
+	g_signal_connect (G_OBJECT (data->combo_box),
 			  "changed", 
 			  G_CALLBACK (filetype_combobox_changed_cb),
-			  file_sel);
+			  data);
 
 	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
 	gtk_widget_show_all (file_sel);
@@ -767,6 +869,16 @@ activate_action_delete_folder_from_sidebar (GtkAction *action,
 }
 
 
+void
+activate_action_find (GtkAction *action, 
+		      gpointer   data)
+{
+	FrWindow *window = data;
+
+	fr_window_find (window);
+}
+
+
 void
 activate_action_select_all (GtkAction *action,
 			    gpointer   data)
@@ -898,6 +1010,42 @@ activate_action_last_output (GtkAction *action,
 }
 
 
+void
+activate_action_go_back (GtkAction *action,
+			 gpointer   data)
+{
+	FrWindow *window = data;
+	fr_window_go_back (window);
+}
+
+
+void
+activate_action_go_forward (GtkAction *action,
+			    gpointer   data)
+{
+	FrWindow *window = data;
+	fr_window_go_forward (window);
+}
+
+
+void
+activate_action_go_up (GtkAction *action,
+		       gpointer   data)
+{
+	FrWindow *window = data;
+	fr_window_go_up_one_level (window);
+}
+
+
+void
+activate_action_go_home (GtkAction *action,
+			 gpointer   data)
+{
+	FrWindow *window = data;
+	fr_window_go_to_location (window, "/", FALSE);
+}
+
+
 void
 activate_action_manual (GtkAction *action,
 			gpointer   data)
