@@ -25,13 +25,9 @@
 #include <math.h>
 #include <unistd.h>
 
+#include <gio/gio.h>
 #include <gtk/gtk.h>
-#include <libgnomeui/gnome-icon-lookup.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs-types.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "file-utils.h"
 #include "fr-stock.h"
@@ -110,12 +106,13 @@ add_clicked_cb (GtkWidget  *widget,
 	char       *tmp;
 	const char *archive_ext;
 	gboolean    do_not_add = FALSE;
+	GError     *error;
 
 	data->add_clicked = TRUE;
 
 	/* Collect data */
 
-	archive_name = gnome_vfs_escape_string (gtk_entry_get_text (GTK_ENTRY (data->a_add_to_entry)));
+	archive_name = g_uri_escape_string (gtk_entry_get_text (GTK_ENTRY (data->a_add_to_entry)), NULL, FALSE);
 
 	/* Check whether the user entered a valid archive name. */
 
@@ -179,7 +176,7 @@ add_clicked_cb (GtkWidget  *widget,
 		return;
 	}
 
-	if (! path_is_dir (archive_dir)) {
+	if (! uri_is_dir (archive_dir)) {
 		GtkWidget *d;
 		int        r;
 		char      *folder_name;
@@ -207,7 +204,7 @@ add_clicked_cb (GtkWidget  *widget,
 		do_not_add = (r != GTK_RESPONSE_YES);
 	}
 
-	if (! do_not_add && ! ensure_dir_exists (archive_dir, 0755)) {
+	if (! do_not_add && ! ensure_dir_exists (archive_dir, 0755, &error)) {
 		GtkWidget  *d;
 
 		d = _gtk_error_dialog_new (GTK_WINDOW (window),
@@ -215,10 +212,11 @@ add_clicked_cb (GtkWidget  *widget,
 					   NULL,
 					   _("Could not create the archive"),
 					   _("Could not create the destination folder: %s."),
-					   gnome_vfs_result_to_string (gnome_vfs_result_from_errno ()));
+					   error->message);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (GTK_WIDGET (d));
 
+		g_error_free (error);
 		g_free (archive_dir);
 		g_free (archive_name);
 		return;
@@ -253,7 +251,7 @@ add_clicked_cb (GtkWidget  *widget,
 	archive_file = g_strconcat (archive_dir, "/", archive_name, NULL);
 	eel_gconf_set_string (PREF_BATCH_ADD_DEFAULT_EXTENSION, archive_ext);
 
-	if (path_is_dir (archive_file)) {
+	if (uri_is_dir (archive_file)) {
 		GtkWidget  *d;
 
 		d = _gtk_error_dialog_new (GTK_WINDOW (window),
@@ -271,7 +269,7 @@ add_clicked_cb (GtkWidget  *widget,
 		return;
 	}
 
-	if (path_is_file (archive_file)) {
+	if (uri_exists (archive_file)) {
 		GtkWidget *d;
 		int        r;
 
@@ -288,8 +286,21 @@ add_clicked_cb (GtkWidget  *widget,
 		r = gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (GTK_WIDGET (d));
 
-		if (r == GTK_RESPONSE_YES)
-			gnome_vfs_unlink (archive_file);
+		if (r == GTK_RESPONSE_YES) {
+			GFile  *file;
+			GError *err = NULL;
+
+			/* FIXME: convert this code in a function in file-utils.c */
+			file = g_file_new_for_uri (archive_file);
+			g_file_delete (file, NULL, &err);
+			if (err != NULL) {
+				g_warning ("Failed to delete file %s: %s", 
+					   archive_file, 
+					   err->message);
+				g_clear_error (&err);
+			}
+			g_object_unref (file);
+		}
 		else {
 			g_free (archive_name);
 			g_free (archive_dir);
@@ -300,7 +311,7 @@ add_clicked_cb (GtkWidget  *widget,
 
 	gtk_widget_destroy (data->dialog);
 
-	if (! path_is_file (archive_file))
+	if (! uri_exists (archive_file))
 		fr_window_archive_new (window, archive_file);
 	else
 		fr_window_archive_open (window, archive_file, GTK_WINDOW (window));
@@ -311,40 +322,6 @@ add_clicked_cb (GtkWidget  *widget,
 }
 
 
-static GdkPixbuf *
-get_pixbuf_from_mime_type (const char *mime_type,
-			   int         icon_size)
-{
-	GtkIconTheme *icon_theme;
-	char         *icon_name = NULL;
-	GdkPixbuf    *icon = NULL;
-
-	icon_theme = gtk_icon_theme_get_default ();
-
-	if (icon_theme == NULL)
-		return NULL;
-
-	icon_name = gnome_icon_lookup (icon_theme,
-				       NULL,
-				       NULL,
-				       NULL,
-				       NULL,
-				       mime_type,
-				       GNOME_ICON_LOOKUP_FLAGS_NONE,
-				       NULL);
-
-	if (icon_name != NULL) 
-		icon = gtk_icon_theme_load_icon (icon_theme,
-						 icon_name,
-						 icon_size,
-						 0,
-						 NULL);
-
-	g_free (icon_name);
-	return icon;
-}
-
-
 static void
 archive_type_combo_box_changed_cb (GtkComboBox *combo_box,
 				   DialogData  *data)
@@ -359,7 +336,7 @@ archive_type_combo_box_changed_cb (GtkComboBox *combo_box,
 		save_type_list =  save_type;
 	mime_type = file_type_desc[save_type_list[idx]].mime_type;
 
-	gtk_image_set_from_pixbuf (GTK_IMAGE (data->add_image), get_pixbuf_from_mime_type (mime_type, ARCHIVE_ICON_SIZE));
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->add_image), get_mime_type_pixbuf (mime_type, ARCHIVE_ICON_SIZE, NULL));
 }
 
 
@@ -419,7 +396,7 @@ dlg_batch_add_files (FrWindow *window,
 
 	data->window = window;
 	data->file_list = file_list;
-	data->single_file = ((file_list->next == NULL) && path_is_file ((char*) file_list->data));
+	data->single_file = ((file_list->next == NULL) && uri_is_file ((char*) file_list->data));
 	data->add_clicked = FALSE;
 
 	/* Get the widgets. */
@@ -443,12 +420,12 @@ dlg_batch_add_files (FrWindow *window,
 	parent = remove_level_from_path (first_filename);
 
 	if (file_list->next == NULL)
-		automatic_name = gnome_vfs_unescape_string (file_name_from_path ((char*) file_list->data), "");
+		automatic_name = g_uri_unescape_string (file_name_from_path ((char*) file_list->data), NULL);
 	else {
-		automatic_name = gnome_vfs_unescape_string (file_name_from_path (parent), "");
+		automatic_name = g_uri_unescape_string (file_name_from_path (parent), NULL);
 		if ((automatic_name == NULL) || (automatic_name[0] == '\0')) {
 			g_free (automatic_name);
-			automatic_name = gnome_vfs_unescape_string (file_name_from_path (first_filename), "");
+			automatic_name = g_uri_unescape_string (file_name_from_path (first_filename), NULL);
 		}
 	}
 
