@@ -26,7 +26,6 @@
 
 #include <glade/glade.h>
 
-#include "bookmarks.h"
 #include "file-utils.h"
 #include "fr-stock.h"
 #include "main.h"
@@ -38,7 +37,9 @@
 
 typedef struct {
 	FrWindow     *window;
-
+	GList        *selected_files;
+	char         *base_dir_for_selection;
+	
 	GtkWidget    *dialog;
 
 	GtkWidget    *e_main_vbox;
@@ -54,8 +55,6 @@ typedef struct {
 
 	GtkWidget    *e_view_folder_checkbutton;
 
-	GtkTooltips  *tooltips;
-
 	gboolean      extract_clicked;
 } DialogData;
 
@@ -69,8 +68,8 @@ destroy_cb (GtkWidget  *widget,
 		fr_window_pop_message (data->window);
 		fr_window_stop_batch (data->window);
 	}
-
-	g_object_unref (data->tooltips);
+	path_list_free (data->selected_files);
+	g_free (data->base_dir_for_selection);
 	g_free (data);
 }
 
@@ -89,7 +88,8 @@ extract_cb (GtkWidget   *w,
 	gboolean    junk_paths;
 	GList      *file_list;
 	char       *password;
-	const char *base_dir = NULL;
+	char       *base_dir = NULL;
+	GError     *error;
 
 	data->extract_clicked = TRUE;
 
@@ -99,7 +99,7 @@ extract_cb (GtkWidget   *w,
 
 	/* check directory existence. */
 
-	if (! path_is_dir (extract_to_dir)) {
+	if (! uri_is_dir (extract_to_dir)) {
 		if (! ForceDirectoryCreation) {
 			GtkWidget *d;
 			int        r;
@@ -129,11 +129,11 @@ extract_cb (GtkWidget   *w,
 				do_not_extract = TRUE;
 		}
 
-		if (! do_not_extract && ! ensure_dir_exists (extract_to_dir, 0755)) {
+		if (! do_not_extract && ! ensure_dir_exists (extract_to_dir, 0755, &error)) {
 			GtkWidget  *d;
 			char       *message;
 
-			message = g_strdup_printf (_("Could not create the destination folder: %s."), gnome_vfs_result_to_string (gnome_vfs_result_from_errno ()));
+			message = g_strdup_printf (_("Could not create the destination folder: %s."), error->message);
 
 			d = _gtk_error_dialog_new (GTK_WINDOW (window),
 						   GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -144,6 +144,7 @@ extract_cb (GtkWidget   *w,
 			gtk_widget_destroy (GTK_WIDGET (d));
 
 			g_free (message);
+			g_error_free (error);
 
 			return FALSE;
 		}
@@ -171,7 +172,7 @@ extract_cb (GtkWidget   *w,
 
 	/* check extraction directory permissions. */
 
-	if (path_is_dir (extract_to_dir)
+	if (uri_is_dir (extract_to_dir)
 	    && ! check_permissions (extract_to_dir, R_OK | W_OK | X_OK)) {
 		GtkWidget *d;
 		char      *utf8_path;
@@ -196,7 +197,7 @@ extract_cb (GtkWidget   *w,
 	fr_window_set_extract_default_dir (window, extract_to_dir, TRUE);
 
 	overwrite = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_overwrite_checkbutton));
-	skip_newer = !gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton)) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton));
+	skip_newer = ! gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton)) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton));
 	junk_paths = ! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_recreate_dir_checkbutton));
 
 	eel_gconf_set_boolean (PREF_EXTRACT_OVERWRITE, overwrite);
@@ -223,9 +224,10 @@ extract_cb (GtkWidget   *w,
 
 	file_list = NULL;
 
-	if (selected_files)
-		file_list = fr_window_get_file_list_selection (window, TRUE, NULL);
-
+	if (selected_files) {
+		file_list = data->selected_files;
+		data->selected_files = NULL;       /* do not the list when destroying the dialog. */
+	} 
 	else if (pattern_files) {
 		const char *pattern;
 
@@ -239,6 +241,13 @@ extract_cb (GtkWidget   *w,
 		}
 	}
 
+	if (selected_files) {
+		base_dir = data->base_dir_for_selection;
+		data->base_dir_for_selection = NULL;
+	}
+	else
+		base_dir = NULL;
+
 	/* close the dialog. */
 
 	gtk_widget_destroy (data->dialog);
@@ -248,9 +257,6 @@ extract_cb (GtkWidget   *w,
 	fr_window_view_folder_after_extract (window, eel_gconf_get_boolean (PREF_EXTRACT_VIEW_FOLDER, FALSE));
 	fr_window_set_password (window, password);
 
-	if (selected_files)
-		base_dir = fr_window_get_current_location (window);
-
 	fr_window_archive_extract (window,
 				   file_list,
 				   extract_to_dir,
@@ -262,6 +268,7 @@ extract_cb (GtkWidget   *w,
 
 	path_list_free (file_list);
 	g_free (extract_to_dir);
+	g_free (base_dir);
 	g_free (password);
 
 	return TRUE;
@@ -377,7 +384,7 @@ create_extra_widget (DialogData *data)
 	gtk_table_attach (GTK_TABLE (table1), data->e_files_entry, 1, 2, 2, 3,
 			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 			  (GtkAttachOptions) (0), 0, 0);
-	gtk_tooltips_set_tip (data->tooltips, data->e_files_entry, _("example: *.txt; *.doc"), NULL);
+	gtk_widget_set_tooltip_text (data->e_files_entry, _("example: *.txt; *.doc"));
 	gtk_entry_set_activates_default (GTK_ENTRY (data->e_files_entry), TRUE);
 
 	data->e_all_radiobutton = gtk_radio_button_new_with_mnemonic (NULL, _("_All files"));
@@ -448,23 +455,21 @@ create_extra_widget (DialogData *data)
 }
 
 
-void
-dlg_extract (GtkWidget *widget,
-	     gpointer   callback_data)
+static void
+dlg_extract__common (FrWindow *window,
+	             GList    *selected_files,
+	             char     *base_dir_for_selection)
 {
-	FrWindow   *window = callback_data;
 	DialogData *data;
 	GtkWidget  *file_sel;
 
 	data = g_new0 (DialogData, 1);
 
 	data->window = window;
+	data->selected_files = selected_files;
+	data->base_dir_for_selection = base_dir_for_selection;
 	data->extract_clicked = FALSE;
 
-	data->tooltips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (data->tooltips));
-	gtk_object_sink (GTK_OBJECT (data->tooltips));
-
 	data->dialog = file_sel =
 		gtk_file_chooser_dialog_new (_("Extract"),
 					     GTK_WINDOW (data->window),
@@ -486,7 +491,7 @@ dlg_extract (GtkWidget *widget,
 
 	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (file_sel), fr_window_get_extract_default_dir (window));
 
-	if (fr_window_get_n_selected_files (window) > 0)
+	if (data->selected_files != NULL)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->e_selected_radiobutton), TRUE);
 	else {
 		gtk_widget_set_sensitive (data->e_selected_radiobutton, FALSE);
@@ -539,9 +544,27 @@ dlg_extract (GtkWidget *widget,
 }
 
 
+void
+dlg_extract (GtkWidget *widget,
+	     gpointer   callback_data)
+{
+	FrWindow *window = callback_data;
+	GList    *files;
+	char     *base_dir;
+	
+	files = fr_window_get_selection (window, FALSE, &base_dir);
+	dlg_extract__common (window, files, base_dir);
+}
+
+
 void
 dlg_extract_folder_from_sidebar (GtkWidget *widget,
 	     			 gpointer   callback_data)
 {
-	/* FIXME */
+	FrWindow *window = callback_data;
+	GList    *files;
+	char     *base_dir;
+	
+	files = fr_window_get_selection (window, TRUE, &base_dir);
+	dlg_extract__common (window, files, base_dir);
 }
