@@ -326,12 +326,7 @@ fr_command_tar_add (FrCommand     *comm,
 		g_free (e_base_dir);
 	}
 
-	if (! g_file_test (comm->filename, G_FILE_TEST_EXISTS))
-		fr_process_add_arg (comm->process, "-cf");
-	else if (update)
-		fr_process_add_arg (comm->process, "-uf");
-	else
-		fr_process_add_arg (comm->process, "-rf");
+	fr_process_add_arg (comm->process, "-rf");
 
 	fr_process_add_arg (comm->process, c_tar->uncomp_filename);
 	fr_process_add_arg (comm->process, "--");
@@ -724,8 +719,8 @@ fr_command_tar_uncompress (FrCommand *comm)
 	}
 
 	{
-		char *uri = get_uri_from_local_path (comm->filename);
-		archive_exists = path_is_file (uri);
+		char *uri = g_filename_to_uri (comm->filename, NULL, NULL);
+		archive_exists = uri_exists (uri);
 		g_free (uri);
 	}
 
@@ -877,7 +872,7 @@ fr_command_tar_init (FrCommand *comm)
 	comm->file_type = FR_FILE_TYPE_TAR;
 
 	comm->propCanModify                = TRUE;
-	comm->propAddCanUpdate             = TRUE;
+	comm->propAddCanUpdate             = FALSE;
 	comm->propAddCanReplace            = FALSE;
 	comm->propAddCanStoreFolders       = TRUE;
 	comm->propExtractCanAvoidOverwrite = TRUE;
