@@ -167,7 +167,7 @@ list__process_line (char     *line,
 
 	/* ignore lines that do not describe a file or a
 	 * directory. */
-	if ((line[0] != 'd') && (line[0] != '-'))
+	if ((line[0] != '?') && (line[0] != 'd') && (line[0] != '-'))
 		return;
 
 	/**/
@@ -222,7 +222,7 @@ add_password_arg (FrCommand     *comm,
 		  const char    *password,
 		  gboolean       always_specify)
 {
-	if (always_specify || ((password != NULL) && (*password != 0))) {
+	if (always_specify || ((password != NULL) && (password[0] != '\0'))) {
 		char *arg;
 		char *e_password;
 
