@@ -26,7 +26,6 @@
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 #include "glib-utils.h"
-#include "utf8-fnmatch.h"
 
 
 #define MAX_PATTERNS 128
@@ -317,32 +316,46 @@ g_utf8_strchomp (char *string)
 }
 
 
-gboolean
-match_patterns (char       **patterns,
-		const char  *string,
-		int          flags)
-{
-	int i;
-	int result;
+#define g_utf8_strstrip(string)    g_utf8_strchomp (g_utf8_strchug (string))
 
-	if (patterns[0] == NULL)
+
+gboolean
+match_regexps (GRegex           **regexps,
+	       const char        *string,
+	       GRegexMatchFlags   match_options)
+{
+	gboolean matched;
+	int      i;
+	
+	if ((regexps == NULL) || (regexps[0] == NULL))
 		return TRUE;
 
 	if (string == NULL)
 		return FALSE;
-
-	result = FNM_NOMATCH;
-	i = 0;
-	while ((result != 0) && (patterns[i] != NULL)) {
-		result = g_utf8_fnmatch (patterns[i], string, flags);
-		i++;
-	}
-
-	return (result == 0);
+	
+	matched = FALSE;
+	for (i = 0; regexps[i] != NULL; i++)
+		if (g_regex_match (regexps[i], string, match_options, NULL)) {
+			matched = TRUE;
+			break;
+		}
+		
+	return matched;
 }
 
 
-#define g_utf8_strstrip(string)    g_utf8_strchomp (g_utf8_strchug (string))
+void
+free_regexps (GRegex **regexps)
+{
+	int i;
+	
+	if (regexps == NULL) 
+		return;
+		
+	for (i = 0; regexps[i] != NULL; i++)
+		g_regex_unref (regexps[i]);
+	g_free (regexps);
+}
 
 
 char **
@@ -351,14 +364,49 @@ search_util_get_patterns (const char *pattern_string)
 	char **patterns;
 	int    i;
 
+	if (pattern_string == NULL)
+		return NULL;
+
 	patterns = g_utf8_strsplit (pattern_string, ";", MAX_PATTERNS);
-	for (i = 0; patterns[i] != NULL; i++)
-		patterns[i] = g_utf8_strstrip (patterns[i]);
+	for (i = 0; patterns[i] != NULL; i++) {
+		char *p1, *p2;
+		
+		p1 = g_utf8_strstrip (patterns[i]);
+		p2 = str_substitute (p1, ".", "\\.");
+		patterns[i] = str_substitute (p2, "*", ".*");
+		
+		g_free (p2);
+		g_free (p1);
+	}
 
 	return patterns;
 }
 
 
+GRegex **
+search_util_get_regexps (const char         *pattern_string,
+			 GRegexCompileFlags  compile_options)
+{
+	char   **patterns;
+	GRegex **regexps;
+	int      i;
+		
+	patterns = search_util_get_patterns (pattern_string);
+	if (patterns == NULL)
+		return NULL;
+		
+	regexps = g_new0 (GRegex*, n_fields (patterns) + 1);
+	for (i = 0; patterns[i] != NULL; i++) 
+		regexps[i] = g_regex_new (patterns[i], 
+					  G_REGEX_OPTIMIZE | compile_options, 
+					  G_REGEX_MATCH_NOTEMPTY, 
+					  NULL);
+	g_strfreev (patterns);
+	
+	return regexps;
+}
+
+
 char *
 _g_strdup_with_max_size (const char *s,
 			 int         max_size)
@@ -559,3 +607,28 @@ g_ptr_array_reverse (GPtrArray *array)
 		g_ptr_array_index (array, j) = tmp;
 	}
 }
+
+
+GHashTable *static_strings = NULL;
+
+
+const char *
+get_static_string (const char *s)
+{
+        const char *result;
+
+        if (s == NULL)
+                return NULL;
+
+        if (static_strings == NULL)
+                static_strings = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
+        if (! g_hash_table_lookup_extended (static_strings, s, (gpointer*) &result, NULL)) {
+                result = g_strdup (s);
+                g_hash_table_insert (static_strings,
+                                     (gpointer) result,
+                                     GINT_TO_POINTER (1));
+        }
+
+        return result;
+}
