@@ -51,6 +51,7 @@ typedef enum {
 	FR_COMPRESS_PROGRAM_BZIP,
 	FR_COMPRESS_PROGRAM_BZIP2,
 	FR_COMPRESS_PROGRAM_COMPRESS,
+	FR_COMPRESS_PROGRAM_LZMA,
 	FR_COMPRESS_PROGRAM_LZOP
 } FRCompressProgram;
 
@@ -93,6 +94,7 @@ typedef enum {
 	FR_FILE_TYPE_GZIP,
 	FR_FILE_TYPE_JAR,
 	FR_FILE_TYPE_LHA,
+	FR_FILE_TYPE_LZMA,
 	FR_FILE_TYPE_LZOP,
 	FR_FILE_TYPE_RAR,
 	FR_FILE_TYPE_RPM,
@@ -100,6 +102,7 @@ typedef enum {
 	FR_FILE_TYPE_TAR_BZ,
 	FR_FILE_TYPE_TAR_BZ2,
 	FR_FILE_TYPE_TAR_GZ,
+	FR_FILE_TYPE_TAR_LZMA,
 	FR_FILE_TYPE_TAR_LZOP,
 	FR_FILE_TYPE_TAR_COMPRESS,
 	FR_FILE_TYPE_STUFFIT,
@@ -115,6 +118,7 @@ typedef struct {
 	char       *ext;
 	char       *mime_type;
 	char       *name;
+	gboolean    supports_password;
 } FRFileTypeDescription;
 
 typedef struct {
